# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.2.1

import enum

import winrt.runtime._internals
import winrt.system
from winrt._winrt_windows_devices_lights_effects import (
    LampArrayBitmapEffect,
    LampArrayBitmapRequestedEventArgs,
    LampArrayBlinkEffect,
    LampArrayColorRampEffect,
    LampArrayCustomEffect,
    LampArrayEffectPlaylist,
    LampArraySolidEffect,
    LampArrayUpdateRequestedEventArgs,
    _ILampArrayEffect,
    ILampArrayEffect,
)

__all__ = [
    "LampArrayEffectCompletionBehavior",
    "LampArrayEffectStartMode",
    "LampArrayRepetitionMode",
    "LampArrayBitmapEffect",
    "LampArrayBitmapRequestedEventArgs",
    "LampArrayBlinkEffect",
    "LampArrayColorRampEffect",
    "LampArrayCustomEffect",
    "LampArrayEffectPlaylist",
    "LampArraySolidEffect",
    "LampArrayUpdateRequestedEventArgs",
    "ILampArrayEffect",
]

class LampArrayEffectCompletionBehavior(enum.IntEnum):
    CLEAR_STATE = 0
    KEEP_STATE = 1

class LampArrayEffectStartMode(enum.IntEnum):
    SEQUENTIAL = 0
    SIMULTANEOUS = 1

class LampArrayRepetitionMode(enum.IntEnum):
    OCCURRENCES = 0
    FOREVER = 1

winrt.runtime._internals.mixin_sequence(LampArrayEffectPlaylist)
